@@ -27,8 +27,7 @@ import quip.field_builder as fb
 import quip.version_builder as vb
 import quip.external as external
 from argparse import RawTextHelpFormatter
-from colorama import Fore, Style, init
-from quip import __version__
+from quip import __version__, yes_or_no, cprint, choose_one
 import keyring
 
 version = __version__
@@ -42,21 +41,10 @@ DOWNLOAD_ACTION = ["download", "pull"]
 UPLOAD_ACTION = ["upload", "push"]
 BUILD_ACTION = ["build", "b", "dist", "zip"]
 CLEAN_ACTION = ["clean", "clear"]
-init()
-
-def cprint(text, color, end='\n'):
-    if len(str(text).strip()) == 0: return
-    fore = getattr(Fore, color.upper())
-    print('{0}{1}{2}'.format(fore, text, Style.RESET_ALL), end=end)
-
-def color(text, color, style='Normal'):
-    fore = getattr(Fore, color.upper())
-    style = getattr(Style, style.upper())
-    return '{0}{1}{2}{3}'.format(fore, style, text, Style.RESET_ALL)
 
 class Quip:
     def __init__(self, log_level=logging.INFO) -> None:
-        cprint(f"======= QUIP (v.{version}) =======", color="cyan")
+        cprint(f"======= QUIP (v.{version}-BETA) =======", color="cyan")
         logging.basicConfig(level=log_level)
         self.in_project_folder = False
         self.args = self.parse_arguments()
@@ -66,7 +54,7 @@ class Quip:
 
     def parse_arguments(self):
         parser = argparse.ArgumentParser(description='Wrapper for UIP command.', formatter_class=RawTextHelpFormatter)
-        parser.add_argument('--version', action='version', version=f'quip {version}')
+        parser.add_argument('--version', action='version', version=f'quip {version}-BETA')
         subparsers = parser.add_subparsers(dest='action')
         parser.add_argument('--config', '-c', default=None,
                              help='path of the global config. Default is ~/.uip_config.yml')
@@ -518,7 +506,7 @@ class Quip:
         
         self.clone_project(from_project_path, all_files=True, exclude_list=exclude_list)
         if ask_for_upload:
-            answer = self.yes_or_no("Do you want to push the template to controller? ", default=True)
+            answer = yes_or_no("Do you want to push the template to controller? ", default=True)
             if answer == True:
                 self.upload_template()
 
@@ -551,7 +539,7 @@ class Quip:
         payload = self.merge_template_scripts(self.project_name)
         logging.debug(f"Payload = {self.format_json(payload)}")
         
-        answer = self.yes_or_no("Are you updating existing template? ", default=True)
+        answer = yes_or_no("Are you updating existing template? ", default=True)
         if answer == True:
             logging.info("Updating existing template")
             result = requests.put(template_url, json=payload, auth=(uac_user, uac_pass), verify=False)
@@ -667,7 +655,7 @@ class Quip:
                 template_dict = fb.dump_template_fields(_json)
                 fields_dict = fb.dump_fields(_json.get("fields"))
                 template_dict["fields"] = fields_dict
-                events_dict = fb.dump_events(_json.get("events"))
+                events_dict = fb.dump_events(_json.get("events", None))
                 template_dict["events"] = events_dict
                 yaml_dump = yaml.dump(template_dict, Dumper=MyDumper, default_flow_style=False, sort_keys=False, width=1000)
                 yaml_dump = yaml_dump.replace('fields:\n', '\nfields:')
@@ -683,12 +671,89 @@ class Quip:
             sys.exit(1)
 
     def create_external_systems(self):
-        sq_config = self._global_conf_external.get("sonarqube", {})
-        sq_url = sq_config.get("url", None)
-        if sq_url is None:
-            logging.warn("Sonarqube configuration is missing in the uip_config file.")
-            return False
+        repository_name = None
+        gl_config = self._global_conf_external.get("gitlab", None)
+        jnks_config = self._global_conf_external.get("jenkins", None)
+        sq_config = self._global_conf_external.get("sonarqube", None)
+        if gl_config is None:
+            logging.warn("Gitlab configuration is missing in the uip_config file.")
+        if jnks_config is None:
+            logging.warn("Jenkins configuration is missing in the uip_config file.")
+        if sq_config is None:
+            logging.warn("SonarQube configuration is missing in the uip_config file.")
+        
+        if gl_config is not None:
+            cprint("\n==== GITLAB SETUP ====", "blue")
+            gl = self.initialize_gitlab(gl_config)
+            self.setup_gitlab(gl, gl_config)
+            repository_name = gl.repository_name
 
+        if repository_name is None:
+            # can not continue without repository_name
+            logging.warn("Cannot continue without GitLab repository.")
+            sys.exit(0)
+
+        if jnks_config is not None:
+            cprint("\n==== JENKINS SETUP ====", "blue")
+            jnks = self.initialize_jenkins(jnks_config)
+
+            gl_groups = gl_config.get("groups", [])
+            jnks_groups = jnks_config.get("groups", {})
+            jnks_group_name, jnks_repository_name = external.parse_repository_full_path(repository_name, gl_groups, jnks_groups)
+            
+            self.setup_jenkins(jnks, jnks_group_name, jnks_repository_name)
+            
+        # create webhook for Jenkins
+        if gl_config is not None and jnks_config is not None:
+            jenkins_url = jnks.get_url(jnks_group_name, jnks_repository_name)
+            gl.create_webhook(jenkins_url)
+
+        if sq_config is not None:
+            cprint("\n==== SONARQUBE SETUP ====", "blue")
+            # check if SonarQube project exists
+            sq_groups = sq_config.get("groups", {})
+            if jnks_group_name in sq_groups:
+                sq_group_name = sq_groups.get(jnks_group_name)
+                if yes_or_no("Do you want to create SonarQube projects?", default=True):
+                    self.create_sonarqube(sq_group_name, jnks_repository_name, sq_config)
+            else:
+                cprint("No need to create SonarQube Project.", "green")
+    
+    def setup_gitlab(self, gl, gl_config):
+        repository_name = None
+        
+        git_path = template = self.join_path(".git", "config")
+        if os.path.exists(git_path):
+            repository_name = external.get_git_info(git_path, "origin")
+        
+        if repository_name is not None:
+            cprint(f"Repository exists in git config: {repository_name}", "magenta")
+
+            if self.check_gitlab_repository_exists(gl, repository_name):
+                cprint(f"Repository exists in GitLab: {repository_name}", "green")
+            else:
+                cprint(f"Repository doesn't exist in GitLab: {repository_name}", "red")
+                if yes_or_no("Do you want to create GitLab repository?", default=True):
+                    repository_name = self.create_gitlab(gl, repository_name)
+        else:
+            if yes_or_no("Do you want to create Gitlab repository?", default=True):
+                repository_name = self.create_gitlab(gl)
+        
+        gl.repository_name = repository_name
+        return gl
+    
+    def setup_jenkins(self, jnks, jnks_group_name, jnks_repository_name):
+        # Check if Jenkins Pipeline exists
+        if jnks.check_job_exists(jnks_group_name, jnks_repository_name):
+            cprint(f"Job exists in Jenkins: {jnks_group_name}/{jnks_repository_name}", "magenta")
+        else:
+            logging.info(f"Repository doesn't exist and will be created. {jnks_group_name}/{jnks_repository_name}")
+            if yes_or_no("Do you want to create Jenkins Job?", default=True):
+                jnks.create_job(jnks_group_name, jnks_repository_name, self.project_name)
+                cprint(f"Jenkins job created: {jnks_group_name}/{jnks_repository_name}", "green")
+
+    def create_sonarqube(self, group_name, repository_name, sq_config):
+        sq_url = sq_config.get("url", None)
         use_token = sq_config.get("use_token", True)
         if use_token:
             sq_pass = self.ask_password(sq_url, "token", prompt="Please enter Personal Access Key: ")
@@ -704,10 +769,83 @@ class Quip:
         
         ssl_verify = sq_config.get("ssl_verify", True)
         sq = external.SonarQube(sq_url, (username, sq_pass), ssl_verify)
-        prefix = sq_config.get("prefix", "cs-uac")
-        sq_project_name = prefix + "-" + self.extension_name
+        prefix = group_name
+        sq_project_name = prefix + "-" + repository_name
         sq.create_project(sq_project_name)
 
+    def create_gitlab(self, gl, repository_name=None):
+        logging.info("Creating GitLab Repository.")
+        
+        if repository_name is None:
+            groups = gl.get_groups()
+            group = choose_one(groups, title="Gitlab Groups", default=gl.default_group)
+            group_id = group[1]
+            _group = gl.gl.groups.get(group_id)
+            group_path = _group.path
+            repository_name = f"{group_path}/{self.extension_name}"
+        
+        if self.check_gitlab_repository_exists(gl, repository_name):
+            cprint(f"There is a repository already exists in GitLab. ({repository_name})", "yellow")
+            cprint(f"Run `git remote add origin {gl.base_url}/{repository_name}.git", "yellow")
+        else:
+            gl.create_project(self.extension_name, group_id)
+            cprint(f"Repository created {repository_name}", "green")
+            cprint(f"Run `git init`", "green")
+            cprint(f"Run `git remote add origin {gl.base_url}/{repository_name}.git`", "green")
+        
+        return repository_name
+    
+    def check_gitlab_repository_exists(self, gl, repository_name):
+        logging.info("Checking GitLab if the repository exists.")
+            
+        projects = gl.get_projects()
+        for project in projects:
+            if repository_name == project[0]:
+                gl.project_id = project[1]
+                return True
+        
+        return False
+
+    def initialize_gitlab(self, gl_config):
+        gl_url = gl_config.get("url", None)
+        gl_token = self.ask_password(gl_url, "token", prompt="Please enter Personal Access Key: ")
+        ssl_verify = gl_config.get("ssl_verify", True)
+        default_group = gl_config.get("default_group", None)
+        gl = external.GitLab(gl_url, gl_token, ssl_verify, default_group)
+        return gl
+
+    
+    def initialize_jenkins(self, jnks_config):
+        logging.info("Connecting to Jenkins Server.")
+        jnks_url = jnks_config.get("url", None)
+        username = jnks_config.get("username", True)
+        ssl_verify = jnks_config.get("ssl_verify", True)
+        jnks_token = self.ask_password(jnks_url, username, prompt=f"Please enter Jenkins Password for ({username}): ")
+        jnks = external.Jenkins(jnks_url, (username, jnks_token), ssl_verify)
+        return jnks
+
+    
+    def create_jenkins(self, repository_name, jnks=None):
+        if jnks is None:
+            jnks_config = self._global_conf_external.get("gitlab", {})
+            jnks_url = jnks_config.get("url", None)
+            if jnks_url is None:
+                logging.warn("Jenkins configuration is missing in the uip_config file.")
+                return None
+
+            
+            username = jnks_config.get("username", True)
+            jnks_token = self.ask_password(jnks_url, username, prompt=f"Please enter Jenkins Password for ({username}): ")
+            ssl_verify = jnks_config.get("ssl_verify", True)
+            default_group = jnks_config.get("default_group", None)
+            jnks = external.Jenkins(jnks_url, (username, jnks_token), ssl_verify, default_group)
+        projects = jnks.get_projects()
+        for project in projects:
+            if repository_name == project[0]:
+                return True
+        
+        return False
+
     def build_zip(self, project_name):
         with tempfile.TemporaryDirectory() as tmpdirname:
             logging.debug(f"Created temporary directory {tmpdirname}")
@@ -748,30 +886,10 @@ class Quip:
         else:
             new_version = forced_version
         cprint(f"NEW Version will be {new_version}", color="green")
-        answer = self.yes_or_no(f"Do you want to update the versions from {current_version} to {new_version}? ", default=True)
+        answer = yes_or_no(f"Do you want to update the versions from {current_version} to {new_version}? ", default=True)
         if answer == True:
             vb.update_version(current_version, new_version, version_files=self._version_files)
 
-
-    def yes_or_no(self, question, default=None):
-        if default == True:
-            options = "(Y/n)"
-        elif default == False:
-            options = "(y/N)"
-        else:
-            options = "(y/n)"
-            default = None
-
-        while True:
-            answer = input(question + options + ": ").lower().strip()
-            
-            if len(answer) == 0 and default is not None:
-                return default
-            elif answer[0] in  ["y", "yes"]:
-                return True
-            elif answer[0] in  ["n", "no"]:
-                return False
-
     def titleize(self, name):
         if name[:3] in ["ue-", "ut-"]:
             name = name[3:]
@@ -1038,7 +1156,7 @@ class Quip:
                 return
 
             if os.path.exists(file_path):
-                if not self.yes_or_no(f"Do you want to overwrite the script file? ({short_file_path}): ", default=False):
+                if not yes_or_no(f"Do you want to overwrite the script file? ({short_file_path}): ", default=False):
                     logging.info(f"Script file NOT updated.")
                     return None
             
@@ -1090,12 +1208,19 @@ class Quip:
     def ask_password(self, server, user_name, prompt=None):
         if prompt is None:
             prompt = f'''Enter password for {user_name}: '''
+            if self.use_keyring:
+                if (keyring.get_password(server, user_name) is not None):
+                    prompt = f'''Enter password for {user_name} or [Enter] to use the existing password : '''
+        else:
+            if self.use_keyring:
+                if (keyring.get_password(server, user_name) is not None):
+                    print("There is a saved password. To used the saved password just press [Enter]")
         password = getpass(prompt=prompt)
         if len(password) == 0:
             if self.use_keyring:
                 logging.debug("Using password from Keyring")
                 # get password 
-                password= keyring.get_password(server, user_name)
+                password = keyring.get_password(server, user_name)
             else:
                 logging.error("Password is missing.")
                 return None
@@ -1156,7 +1281,7 @@ class QuipGlobalConfig:
     
     def setup_config(self, config_file):
         logging.info("You don't have any config file. I think this is the first time you are running this tool.")
-        if self.yes_or_no(f"Do you want to download sample quip config? (Destination: {config_file}): ", default=True):
+        if yes_or_no(f"Do you want to download sample quip config? (Destination: {config_file}): ", default=True):
             response = requests.get("https://stb-se-dev.s3.amazonaws.com/quip/.uip_config.yml.sample")
             if response.ok:
                 conf = yaml.safe_load(response.text)
